@@ -18,7 +18,8 @@
 #ifndef DEBUG_H
 #define DEBUG_H
 
-#include <tilda-config.h>
+#include "tilda-config.h"
+#include <libgen.h>
 
 #include <glib.h>
 #include <stdio.h>
@@ -64,17 +65,30 @@
 
 #if DEBUG_FUNCTIONS
 
-    #define DEBUG_FUNCTION(NAME) g_printerr("FUNCTION ENTERED: %s\n", (NAME))
+    #define DEBUG_FUNCTION(NAME) g_printerr("%s: FUNCTION ENTERED: %s\n", basename(__FILE__), (NAME))
+
+    #define DEBUG_FUNCTION_MESSAGE(NAME,FORMAT,args...) { \
+        gchar *message = g_strdup_printf ((FORMAT), ##args); \
+        g_printerr("%s: FUNCTION ENTERED: %s: %s", basename(__FILE__), (NAME), message); \
+        g_free (message); \
+   }
 
 #else
 
     #define DEBUG_FUNCTION(NAME) {}
 
+    #define DEBUG_FUNCTION_MESSAGE(NAME,FORMAT,args...) {}
+
 #endif
 
 /* A macro that calls perror() with a consistent header string */
 #define TILDA_PERROR() perror("Tilda encountered an error")
 
+#ifdef DEBUG
+#  define TRACE(x) x
+#else
+#  define TRACE(x) do {} while (FALSE);
+#endif
 
 #endif /* DEBUG_H */
 
