@@ -10,23 +10,48 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
+#define _POSIX_SOURCE /* feature test macro for signal functions */
+#define _XOPEN_SOURCE /* feature test macro for popen */
+
+/*
+ * This message is shown in a modal dialog when tilda starts and there is a problem parsing the configuration file.
+ * Such problems can occur for example if the config file contains a key-value pair that is unknown to tilda. This
+ * can be the case if the user manually modified the configuration file or if a newer version of tilda was run and
+ * which saved some settings that are not known by older versions of tilda.
+ */
+#define TILDA_CONFIG_PARSE_ERROR \
+    "<b>A problem occurred while parsing the config file.</b>\n\n" \
+    "This can happen if the tilda config contains a setting that is unknown to tilda. " \
+    "Tilda will now start with the default configuration."
+
+/*
+ * This message is shown in a modal dialog when tilda starts and there is a any other problem with the configuration
+ * file except a parse error.
+ */
+#define TILDA_CONFIG_OTHER_ERROR \
+    "<b>An unexpected error occured while parsing the config file.</b>\n\n" \
+    "The default configuration will be used instead. This error can occur if the configuration file is corrupted" \
+    "or otherwise unreadable. Tilda will now start with a default configuration."
+
 
 #include <tilda-config.h>
 
-#include <debug.h>
-#include <tilda.h>
-#include <callback_func.h>
-#include <configsys.h>
-#include <tilda_window.h>
-#include <key_grabber.h> /* for pull */
-#include <wizard.h>
-#include <xerror.h>
+#include "debug.h"
+#include "tilda.h"
+#include "callback_func.h"
+#include "configsys.h"
+#include "tilda_window.h"
+#include "key_grabber.h" /* for pull */
+#include "wizard.h"
+#include "xerror.h"
+#include "tomboykeybinder.h"
 
 #include <sys/ioctl.h>
 #include <sys/stat.h>
+#include <sys/file.h>
+#include <errno.h>
 #include <sys/dir.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -44,23 +69,25 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xlibint.h>
-#include <X11/Xproto.h>
-#include <X11/extensions/Xrandr.h>
 
 #include <vte/vte.h>
 #include <glib/gstdio.h>
 
-
-static gchar *create_lock_file (struct lock_info lock)
+/**
+* If lock->pid is 0 then the file is not opened exclusively. Instead flock() must be used to obtain a lock.
+* Otherwise an exclusive lock file is created for the process.
+*/
+static gchar *create_lock_file (struct lock_info *lock)
 {
     DEBUG_FUNCTION ("create_lock_file");
-    DEBUG_ASSERT (lock.instance >= 0);
-    DEBUG_ASSERT (lock.pid >= 1);
+    DEBUG_ASSERT (lock != NULL);
+    DEBUG_ASSERT (lock->instance >= 0);
+    DEBUG_ASSERT (lock->pid >= 0);
 
     gint ret;
     gchar *lock_file_full;
     gchar *lock_dir = g_build_filename (g_get_user_cache_dir (), "tilda", "locks", NULL);
-    gchar *lock_file = g_strdup_printf ("lock_%d_%d", lock.pid, lock.instance);
+    gchar *lock_file = g_strdup_printf ("lock_%d_%d", lock->pid, lock->instance);
 
     /* Make the ~/.cache/tilda/locks directory */
     ret = g_mkdir_with_parents (lock_dir,  S_IRUSR | S_IWUSR | S_IXUSR);
@@ -72,11 +99,17 @@ static gchar *create_lock_file (struct lock_info lock)
     lock_file_full = g_build_filename (lock_dir, lock_file, NULL);
 
     /* Create the lock file */
-    ret = g_creat (lock_file_full, S_IRUSR | S_IWUSR | S_IXUSR);
+    if(lock->pid == 0) {
+        ret = g_open(lock_file_full, O_CREAT, S_IRUSR | S_IWUSR);
+    } else {
+        ret = g_open(lock_file_full, O_WRONLY | O_CREAT | O_EXCL, 0);
+    }
 
     if (ret == -1)
         goto creat_fail;
 
+    lock->file_descriptor = ret;
+
     g_free (lock_file);
     g_free (lock_dir);
 
@@ -191,6 +224,29 @@ nomatch:
 #endif
 }
 
+/**
+* Gets a list of the pids in
+*/
+static GSList *getPids() {
+    GSList *pids = NULL;
+    FILE *ps_output;
+    const gchar ps_command[] = "ps -C tilda -o pid=";
+    gchar buf[16]; /* Really shouldn't need more than 6 */
+
+    if ((ps_output = popen (ps_command, "r")) == NULL) {
+        g_printerr (_("Unable to run command: `%s'\n"), ps_command);
+        return NULL;
+    }
+
+    /* The popen() succeeded, get all of the pids */
+    while (fgets (buf, sizeof(buf), ps_output) != NULL)
+        pids = g_slist_append (pids, GINT_TO_POINTER (atoi (buf)));
+
+    /* We've read all of the pids, exit */
+    pclose (ps_output);
+    return pids;
+}
+
 /**
  * Remove stale lock files from the ~/.tilda/locks/ directory.
  *
@@ -201,30 +257,16 @@ static gint remove_stale_lock_files ()
 {
     DEBUG_FUNCTION ("remove_stale_lock_files");
 
-    GSList *pids = NULL;
-    FILE *ps_output;
-    const gchar ps_command[] = "ps -C tilda -o pid=";
-    gchar buf[16]; /* Really shouldn't need more than 6 */
-
-    if ((ps_output = popen (ps_command, "r")) == NULL)
-    {
-        g_printerr (_("Unable to run command: `%s'\n"), ps_command);
+    GSList *pids = getPids();
+    if(pids == NULL) {
         return -1;
     }
 
-    /* The popen() succeeded, get all of the pids */
-    while (fgets (buf, sizeof(buf), ps_output) != NULL)
-        pids = g_slist_append (pids, GINT_TO_POINTER (atoi (buf)));
-
-    /* We've read all of the pids, exit */
-    pclose (ps_output);
-
     struct lock_info *lock;
     gchar *lock_dir = g_build_filename (g_get_user_cache_dir (), "tilda", "locks", NULL);
     gchar *remove_file;
     gchar *filename;
     GDir *dir;
-    gint pid;
 
     /* Open the lock directory for reading */
     dir = g_dir_open (lock_dir, 0, NULL);
@@ -238,7 +280,7 @@ static gint remove_stale_lock_files ()
 
     /* For each possible lock file, check if it is a lock, and see if
      * it matches one of the running tildas */
-    while (filename = (gchar*) g_dir_read_name (dir))
+    while ((filename = (gchar*) g_dir_read_name (dir)) != NULL)
     {
         lock = islockfile (filename);
 
@@ -256,6 +298,7 @@ static gint remove_stale_lock_files ()
 
     g_dir_close (dir);
     g_free (lock_dir);
+    g_slist_free(pids);
 
     return 0;
 }
@@ -278,11 +321,16 @@ static gboolean parse_cli (int argc, char *argv[])
     gchar *background_color = config_getstr ("background_color");
     gchar *command = config_getstr ("command");
     gchar *font = config_getstr ("font");
-    gchar *image = config_getstr ("image");
     gchar *working_dir = config_getstr ("working_dir");
 
-    gint lines = config_getint ("lines");
+#ifdef VTE_290
+    gchar *image = config_getstr ("image");
     gint transparency = config_getint ("transparency");
+#else
+    gint back_alpha = config_getint ("back_alpha");
+#endif
+
+    gint lines = config_getint ("lines");
     gint x_pos = config_getint ("x_pos");
     gint y_pos = config_getint ("y_pos");
 
@@ -301,12 +349,16 @@ static gboolean parse_cli (int argc, char *argv[])
         { "font",               'f', 0, G_OPTION_ARG_STRING,    &font,              N_("Set the font to the following string"), NULL },
         { "lines",              'l', 0, G_OPTION_ARG_INT,       &lines,             N_("Scrollback Lines"), NULL },
         { "scrollbar",          's', 0, G_OPTION_ARG_NONE,      &scrollbar,         N_("Use Scrollbar"), NULL },
-        { "transparency",       't', 0, G_OPTION_ARG_INT,       &transparency,      N_("Opaqueness: 0-100%"), NULL },
         { "version",            'v', 0, G_OPTION_ARG_NONE,      &version,           N_("Print the version, then exit"), NULL },
         { "working-dir",        'w', 0, G_OPTION_ARG_STRING,    &working_dir,       N_("Set Initial Working Directory"), NULL },
         { "x-pos",              'x', 0, G_OPTION_ARG_INT,       &x_pos,             N_("X Position"), NULL },
         { "y-pos",              'y', 0, G_OPTION_ARG_INT,       &y_pos,             N_("Y Position"), NULL },
+#ifdef VTE_290
         { "image",              'B', 0, G_OPTION_ARG_STRING,    &image,             N_("Set Background Image"), NULL },
+        { "transparency",       't', 0, G_OPTION_ARG_INT,       &transparency,      N_("Opaqueness: 0-100%"), NULL },
+#else
+        { "background-alpha",   't', 0, G_OPTION_ARG_INT,       &back_alpha,        N_("Opaqueness: 0-100%"), NULL },
+#endif
         { "config",             'C', 0, G_OPTION_ARG_NONE,      &show_config,       N_("Show Configuration Wizard"), NULL },
         { NULL }
     };
@@ -316,15 +368,14 @@ static gboolean parse_cli (int argc, char *argv[])
     GError *error = NULL;
     GOptionContext *context = g_option_context_new (NULL);
     g_option_context_add_main_entries (context, cl_opts, NULL);
-    g_option_context_add_group (context, gtk_get_option_group (TRUE));
     g_option_context_parse (context, &argc, &argv, &error);
     g_option_context_free (context);
 
     /* Check for unknown options, and give a nice message if there are some */
     if (error)
     {
-        const char *msg = _("Error parsing command-line options. Try \"tilda --help\"\nto see all possible options.\n\nError message: %s\n");
-        g_printerr (msg, error->message);
+        g_printerr (_("Error parsing command-line options. Try \"tilda --help\"\nto see all possible options.\n\nError message: %s\n"),
+                    error->message);
 
         exit (EXIT_FAILURE);
     }
@@ -348,28 +399,73 @@ static gboolean parse_cli (int argc, char *argv[])
         exit (EXIT_SUCCESS);
     }
 
+    /* This block is only used to initialize the Glib and GTK internal options. That way the users can pass additional command line options,
+     * that are used by Glib and GTK. We do this separate from the above options, because we pass TRUE to the gtk_get_option group function
+     * which causes GTK to initialize the default display. This way it is possible to invoke `tilda --version` without getting an
+     * error if there is no display available.
+     */
+    error = NULL;
+    context = g_option_context_new (NULL);
+    g_option_context_add_group (context, gtk_get_option_group (TRUE));
+    g_option_context_parse (context, &argc, &argv, &error);
+    g_option_context_free (context);
+
+    if (error)
+    {
+        g_printerr (_("Error parsing Glib and GTK specific command-line options. Try \"tilda --help-all\"\nto see all possible options.\n\nError message: %s\n"),
+                    error->message);
+
+        exit (EXIT_FAILURE);
+    }
+
     /* Now set the options in the config, if they changed */
-    if (background_color != config_getstr ("background_color"))
+    if (background_color != config_getstr ("background_color")) {
         config_setstr ("background_color", background_color);
+
+        GdkColor col;
+        if (gdk_color_parse(background_color, &col)) {
+            config_setint("back_red", col.red);
+            config_setint("back_green", col.green);
+            config_setint("back_blue", col.blue);
+        }
+
+        g_free(background_color);
+    }
     if (command != config_getstr ("command"))
     {
         config_setbool ("run_command", TRUE);
         config_setstr ("command", command);
+        g_free(command);
     }
-    if (font != config_getstr ("font"))
+    if (font != config_getstr ("font")) {
         config_setstr ("font", font);
-    if (image != config_getstr ("image"))
+        g_free(font);
+    }
+#ifdef VTE_290
+    if (image != config_getstr ("image")) {
         config_setstr ("image", image);
-    if (working_dir != config_getstr ("working_dir"))
-        config_setstr ("working_dir", working_dir);
-
-    if (lines != config_getint ("lines"))
-        config_setint ("lines", lines);
+        g_free(image);
+    }
     if (transparency != config_getint ("transparency"))
     {
         config_setbool ("enable_transparency", transparency);
         config_setint ("transparency", transparency);
     }
+#else
+    if (back_alpha != config_getint ("back_alpha"))
+    {
+        config_setbool ("enable_transparency", ~back_alpha & 0xffff);
+        config_setint ("back_alpha", back_alpha);
+    }
+#endif
+    if (working_dir != config_getstr ("working_dir")) {
+        config_setstr ("working_dir", working_dir);
+        g_free(working_dir);
+    }
+
+    if (lines != config_getint ("lines"))
+        config_setint ("lines", lines);
+
     if (x_pos != config_getint ("x_pos"))
         config_setint ("x_pos", x_pos);
     if (y_pos != config_getint ("y_pos"))
@@ -416,6 +512,10 @@ static gchar *get_config_file_name (gint instance)
     return config_file;
 }
 
+static gint _cmp_locks(gint a, gint b) {
+  return a - b;
+}
+
 /**
  * get_instance_number ()
  *
@@ -429,11 +529,14 @@ static gint get_instance_number ()
 {
     DEBUG_FUNCTION ("get_instance_number");
 
-    gint i;
     gchar *name;
 
+    GSequence *seq;
+    GSequenceIter *iter;
+    gint lowest_lock_instance = 0;
+    gint current_lock_instance;
+
     GDir *dir;
-    GSList *list = NULL;
     struct lock_info *lock;
     gchar *lock_dir = g_build_filename (g_get_user_cache_dir (), "tilda", "locks", NULL);
 
@@ -449,38 +552,38 @@ static gint get_instance_number ()
     }
 
     /* Look through every file in the lock directory, and see if it is a lock file.
-     * If it is a lock file, store it's instance number in the list. */
-    while (name = (gchar*)g_dir_read_name (dir))
+     * If it is a lock file, insert it in a sorted sequence. */
+    seq = g_sequence_new(NULL);
+    while ((name = (gchar*)g_dir_read_name (dir)) != NULL)
     {
         lock = islockfile (name);
 
         if (lock != NULL)
         {
-            list = g_slist_append (list, GINT_TO_POINTER (lock->instance));
+            g_sequence_insert_sorted(seq, GINT_TO_POINTER(lock->instance), (GCompareDataFunc)_cmp_locks, NULL);
             g_free (lock);
         }
     }
 
     g_dir_close (dir);
-
-    /* Find the lowest available instance.
-     *
-     * This is not the most efficient algorithm ever, but the
-     * list should not be too big, so it's ok for now. */
-    for (i=0; i<INT_MAX; i++)
-        if (!g_slist_find (list, GINT_TO_POINTER (i)))
-            break;
-
     g_free (lock_dir);
-    g_slist_free (list);
 
-    return i;
+    /* We iterate the sorted sequence of lock instances to find the first (lowest) number *not* taken. */
+    for (iter = g_sequence_get_begin_iter(seq); !g_sequence_iter_is_end(iter); iter = g_sequence_iter_next(iter)) {
+      current_lock_instance = GPOINTER_TO_INT(g_sequence_get(iter));
+      if (lowest_lock_instance < current_lock_instance)
+        break;
+      else
+        lowest_lock_instance = current_lock_instance + 1;
+    }
+
+    g_sequence_free(seq);
+
+    return lowest_lock_instance;
 }
 
-static void termination_handler (gint signum)
-{
+static void termination_handler (G_GNUC_UNUSED gint signum) {
     DEBUG_FUNCTION ("termination_handler");
-
     gtk_main_quit ();
 }
 
@@ -488,8 +591,7 @@ static void termination_handler (gint signum)
  * This is to do the migration of config files from ~/.tilda to the
  * XDG_*_HOME folders
  */
-static int migrate_config_files(char *old_config_path)
-{
+static void migrate_config_files(char *old_config_path) {
     gchar* old_lock_dir = g_build_filename(old_config_path, "locks", NULL);
     gchar* new_lock_dir = g_build_filename(g_get_user_cache_dir (), "tilda", "locks", NULL);
     gchar* new_config_dir = g_build_filename(g_get_user_config_dir (), "tilda", NULL);
@@ -513,7 +615,7 @@ static void load_custom_css_file () {
         g_get_user_config_dir (), "tilda", "style.css", NULL);
     if (g_file_test (cssfilename, G_FILE_TEST_EXISTS)) {
         g_print (_("Found style.css in the user config directory, "
-            "applying user css stlye.\n"));
+            "applying user css style.\n"));
         provider = gtk_css_provider_new ();
         gtk_style_context_add_provider_for_screen (
             gdk_screen_get_default(),
@@ -529,9 +631,12 @@ static void load_custom_css_file () {
 
 int main (int argc, char *argv[])
 {
-    DEBUG_FUNCTION ("main");
+    DEBUG_FUNCTION_MESSAGE ("main", "Using libvte version: %i.%i.%i\n",
+                            VTE_MAJOR_VERSION, VTE_MINOR_VERSION, VTE_MICRO_VERSION);
 
-    tilda_window *tw = NULL;
+    tilda_window tw;
+    /* NULL set the tw pointers so we can get a clean exit on initialization failure */
+    memset(&tw, 0, sizeof(tilda_window));
 
     struct sigaction sa;
     struct lock_info lock;
@@ -551,10 +656,41 @@ int main (int argc, char *argv[])
     /* Remove stale lock files */
     remove_stale_lock_files ();
 
+    /* The global lock file is used to synchronize the start up of multiple simultaneously starting tilda processes.
+     * The processes will synchronize on a lock file named lock_0_0, such that the part of determining the instance
+     * number and creating the per process lock file (lock_<pid>_<instance>) is atomic. Without this it could
+     * happen, that a second tilda instance was trying to determine its instance number before the first instance
+     * had finished creating its lock file, this resulted in two processes with the same instance number and could lead
+     * to corruption of the config file.
+     * In order to test if this works the following shell command can be used: "tilda & tilda && fg", which causes
+     * two tilda processes to be started simultaneously. See:
+     *
+     *     http://stackoverflow.com/questions/3004811/how-do-you-run-multiple-programs-from-a-bash-script
+     */
+    struct lock_info global_lock;
+    global_lock.instance = 0;
+    global_lock.pid = 0;
+    gchar *global_lock_file = NULL;
+
+    global_lock_file = create_lock_file(&global_lock);
+    if(global_lock_file == NULL) {
+        perror("Error creating global lock file");
+        return EXIT_FAILURE;
+    }
+    flock(global_lock.file_descriptor, LOCK_EX);
+
+    /* Start of atomic section. */
     lock.pid = getpid ();
     lock.instance = get_instance_number ();
+    lock_file = create_lock_file (&lock);
     config_file = get_config_file_name (lock.instance);
-    lock_file = create_lock_file (lock);
+    /* End of atomic section */
+
+    flock(global_lock.file_descriptor, LOCK_UN);
+    g_remove (global_lock_file);
+    close(lock.file_descriptor);
+    g_free(global_lock_file);
+
 
 #if ENABLE_NLS
     /* Gettext Initialization */
@@ -575,7 +711,7 @@ int main (int argc, char *argv[])
             g_mem_set_vtable (glib_mem_profiler_table);
 #endif
     /* Start up the configuration system */
-    config_init (config_file);
+    gint config_init_result = config_init (config_file);
 
     /* Parse the command line */
     need_wizard = parse_cli (argc, argv);
@@ -583,20 +719,40 @@ int main (int argc, char *argv[])
     /* We're about to startup X, so set the error handler. */
     XSetErrorHandler (xerror_handler);
 
-    /* Initialize GTK and libglade */
+    /* Initialize GTK. Any code that interacts with GTK (e.g. creating a widget) should come after this call. */
     gtk_init (&argc, &argv);
 
+    /* This section shows a modal dialog to notify the user that something has gone wrong when loading the config.
+     * Earlier version only used to print a message to stderr, but since tilda is usually not started from a
+     * console this message would have been lost and a message dialog is much more user friedly.
+     */
+    GtkWidget *dialog = NULL;
+    if(config_init_result == 1) {
+        dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                                        TILDA_CONFIG_PARSE_ERROR);
+    } else if (config_init_result != 0) {
+        dialog = gtk_message_dialog_new_with_markup(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
+                                        TILDA_CONFIG_OTHER_ERROR);
+    }
+
+    if(dialog) {
+        g_message("Running Dialog");
+        gtk_dialog_run(GTK_DIALOG(dialog));
+        gtk_widget_destroy(dialog);
+    }
+
     load_custom_css_file ();
 
     /* create new tilda_window */
-    tw = tilda_window_init (config_file, lock.instance);
+    gboolean success = tilda_window_init (config_file, lock.instance, &tw);
 
-    /* Check the allocations above */
-    if (tw == NULL)
-        goto tw_alloc_failed;
+    if(!success) {
+        fprintf(stderr, "tilda.c: initialization failed\n");
+        goto initialization_failed;
+    }
 
     /* Adding widget title for CSS selection */
-    gtk_widget_set_name (GTK_WIDGET(tw->window), "Main");
+    gtk_widget_set_name (GTK_WIDGET(tw.window), "Main");
 
     /* Initialize and set up the keybinding to toggle tilda's visibility. */
     tomboy_keybinder_init ();
@@ -613,10 +769,10 @@ int main (int argc, char *argv[])
     sigaction (SIGKILL, &sa, NULL);
 
     /* If the config file doesn't exist open up the wizard */
-    if (access (tw->config_file, R_OK) == -1)
+    if (access (tw.config_file, R_OK) == -1)
     {
         /* We probably need a default key, too ... */
-        gchar *default_key = g_strdup_printf ("F%d", tw->instance+1);
+        gchar *default_key = g_strdup_printf ("F%d", tw.instance+1);
         config_setstr ("key", default_key);
         g_free (default_key);
 
@@ -628,34 +784,34 @@ int main (int argc, char *argv[])
      * Note that the key will be bound upon exiting the wizard */
     if (need_wizard) {
         g_print ("Starting the wizard to configure tilda options.");
-        wizard (tw);
+        wizard (&tw);
     } else {
-        gint ret = tilda_keygrabber_bind (config_getstr ("key"), tw);
+        gint ret = tilda_keygrabber_bind (config_getstr ("key"), &tw);
 
         if (!ret)
         {
             /* The key was unbindable, so we need to show the wizard */
             show_invalid_keybinding_dialog (NULL, _("The keybinding you chose for \"Pull Down Terminal\" is invalid. Please choose another."));
-            wizard (tw);
+            wizard (&tw);
         }
     }
 
-    pull (tw, config_getbool ("hidden") ? PULL_UP : PULL_DOWN);
+    pull (&tw, config_getbool ("hidden") ? PULL_UP : PULL_DOWN, FALSE);
 
     g_print ("Tilda has started. Press %s to pull down the window.\n",
         config_getstr ("key"));
     /* Whew! We're finally all set up and ready to run GTK ... */
     gtk_main();
 
-    /* Ok, we're at the end of our run. Time to clean up ... */
-    tilda_window_free (tw);
+initialization_failed:
+    tilda_window_free(&tw);
 
-tw_alloc_failed:
+    /* Ok, we're at the end of our run. Time to clean up ... */
     config_free (config_file);
     g_remove (lock_file);
+    close(lock.file_descriptor);
     g_free (lock_file);
     g_free (config_file);
-
     return 0;
 }
 
