@@ -10,14 +10,13 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef TILDA_H
 #define TILDA_H
 
-#include <tilda_window.h>
+#include "tilda_window.h"
 
 #include <gtk/gtk.h>
 
@@ -32,9 +31,26 @@ struct lock_info
 {
     gint pid;
     gint instance;
+    gint file_descriptor;
 };
 
+#define GUINT16_TO_FLOAT(color) (color / (double) 0xFFFF)
+#define GUINT16_FROM_FLOAT(value) ((int) (value * 0xFFFF + 0.5))
+
+#define RGB(r,g,b) (r) / (gdouble)G_MAXUINT16, \
+                     (g) / (gdouble)G_MAXUINT16, \
+                     (b) / (gdouble)G_MAXUINT16, 1.
 
 G_END_DECLS
 
 #endif
+
+/* Future compatibility with VTE-2.90 */
+#ifndef VTE_290 
+#define vte_terminal_set_colors_rgba vte_terminal_set_colors
+#define vte_terminal_set_color_background_rgba vte_terminal_set_color_background
+#define vte_terminal_set_color_foreground_rgba vte_terminal_set_color_foreground
+#define vte_terminal_set_color_cursor_rgba vte_terminal_set_color_cursor
+#define VteTerminalCursorShape VteCursorShape
+#endif
+
