@@ -1,3 +1,26 @@
+/* tomboykeybinder.c
+* Copyright (C) 2008 Alex Graveley
+*
+* Permission is hereby granted, free of charge, to any person obtaining
+* a copy of this software and associated documentation files (the
+* "Software"), to deal in the Software without restriction, including
+* without limitation the rights to use, copy, modify, merge, publish,
+* distribute, sublicense, and/or sell copies of the Software, and to
+* permit persons to whom the Software is furnished to do so, subject to
+* the following conditions:
+*
+* The above copyright notice and this permission notice shall be
+* included in all copies or substantial portions of the Software.
+*
+* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+* LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+* OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+* WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+*/
+
 #include <string.h>
 
 #include <gdk/gdk.h>
@@ -6,22 +29,14 @@
 
 #include "eggaccelerators.h"
 #include "tomboykeybinder.h"
-
-/* Uncomment the next line to print a debug trace. */
-/* #define DEBUG */
-
-#ifdef DEBUG
-#  define TRACE(x) x
-#else
-#  define TRACE(x) do {} while (FALSE);
-#endif
+#include "debug.h"
 
 typedef struct _Binding {
 	TomboyBindkeyHandler  handler;
 	gpointer              user_data;
 	char                 *keystring;
-	uint                  keycode;
-	uint                  modifiers;
+	unsigned int          keycode;
+	unsigned int          modifiers;
 } Binding;
 
 static GSList *bindings = NULL;
@@ -33,21 +48,21 @@ static guint num_lock_mask, caps_lock_mask, scroll_lock_mask;
 static void
 lookup_ignorable_modifiers (GdkKeymap *keymap)
 {
-	egg_keymap_resolve_virtual_modifiers (keymap, 
+	egg_keymap_resolve_virtual_modifiers (keymap,
 					      EGG_VIRTUAL_LOCK_MASK,
 					      &caps_lock_mask);
 
-	egg_keymap_resolve_virtual_modifiers (keymap, 
+	egg_keymap_resolve_virtual_modifiers (keymap,
 					      EGG_VIRTUAL_NUM_LOCK_MASK,
 					      &num_lock_mask);
 
-	egg_keymap_resolve_virtual_modifiers (keymap, 
+	egg_keymap_resolve_virtual_modifiers (keymap,
 					      EGG_VIRTUAL_SCROLL_LOCK_MASK,
 					      &scroll_lock_mask);
 }
 
 static void
-grab_ungrab_with_ignorable_modifiers (GdkWindow *rootwin, 
+grab_ungrab_with_ignorable_modifiers (GdkWindow *rootwin,
 				      Binding   *binding,
 				      gboolean   grab)
 {
@@ -61,27 +76,27 @@ grab_ungrab_with_ignorable_modifiers (GdkWindow *rootwin,
 		caps_lock_mask | scroll_lock_mask,
 		num_lock_mask  | caps_lock_mask | scroll_lock_mask,
 	};
-	int i;
+	guint i;
 
 	for (i = 0; i < G_N_ELEMENTS (mod_masks); i++) {
 		if (grab) {
-			XGrabKey (GDK_WINDOW_XDISPLAY (rootwin), 
-				  binding->keycode, 
-				  binding->modifiers | mod_masks [i], 
+			XGrabKey (GDK_WINDOW_XDISPLAY (rootwin),
+				  binding->keycode,
+				  binding->modifiers | mod_masks [i],
 				  GDK_WINDOW_XID (rootwin),
-				  False, 
+				  False,
 				  GrabModeAsync,
 				  GrabModeAsync);
 		} else {
 			XUngrabKey (GDK_WINDOW_XDISPLAY (rootwin),
 				    binding->keycode,
-				    binding->modifiers | mod_masks [i], 
+				    binding->modifiers | mod_masks [i],
 				    GDK_WINDOW_XID (rootwin));
 		}
 	}
 }
 
-static gboolean 
+static gboolean
 do_grab_key (Binding *binding)
 {
 	GdkKeymap *keymap = gdk_keymap_get_default ();
@@ -93,14 +108,14 @@ do_grab_key (Binding *binding)
 	if (keymap == NULL || rootwin == NULL)
 		return FALSE;
 
-	if (!egg_accelerator_parse_virtual (binding->keystring, 
-					    &keysym, 
+	if (!egg_accelerator_parse_virtual (binding->keystring,
+					    &keysym,
 					    &virtual_mods))
 		return FALSE;
 
 	TRACE (g_print ("Got accel %d, %d\n", keysym, virtual_mods));
 
-	binding->keycode = XKeysymToKeycode (GDK_WINDOW_XDISPLAY (rootwin), 
+	binding->keycode = XKeysymToKeycode (GDK_WINDOW_XDISPLAY (rootwin),
 					     keysym);
 	if (binding->keycode == 0)
 		return FALSE;
@@ -115,8 +130,8 @@ do_grab_key (Binding *binding)
 
 	gdk_error_trap_push ();
 
-	grab_ungrab_with_ignorable_modifiers (rootwin, 
-					      binding, 
+	grab_ungrab_with_ignorable_modifiers (rootwin,
+					      binding,
 					      TRUE /* grab */);
 
 	gdk_flush ();
@@ -129,73 +144,72 @@ do_grab_key (Binding *binding)
 	return TRUE;
 }
 
-static gboolean 
+static gboolean
 do_ungrab_key (Binding *binding)
 {
 	GdkWindow *rootwin = gdk_get_default_root_window ();
 
 	TRACE (g_print ("Removing grab for '%s'\n", binding->keystring));
 
-	grab_ungrab_with_ignorable_modifiers (rootwin, 
-					      binding, 
+	grab_ungrab_with_ignorable_modifiers (rootwin,
+					      binding,
 					      FALSE /* ungrab */);
 
 	return TRUE;
 }
 
 static GdkFilterReturn
-filter_func (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
-{
-	GdkFilterReturn return_val = GDK_FILTER_CONTINUE;
-	XEvent *xevent = (XEvent *) gdk_xevent;
-	guint event_mods;
-	GSList *iter;
+filter_func (GdkXEvent *gdk_xevent, G_GNUC_UNUSED GdkEvent *event, G_GNUC_UNUSED gpointer data) {
+    XEvent *xevent = (XEvent *) gdk_xevent;
+    guint event_mods;
+    GSList *iter;
 
-	TRACE (g_print ("Got Event! %d, %d\n", xevent->type, event->type));
+    switch (xevent->type) {
+        case KeyPress:
+            TRACE (g_print ("Got KeyPress! keycode: %d, modifiers: %d\n",
+                    xevent->xkey.keycode,
+                    xevent->xkey.state));
 
-	switch (xevent->type) {
-	case KeyPress:
-		TRACE (g_print ("Got KeyPress! keycode: %d, modifiers: %d\n", 
-				xevent->xkey.keycode, 
-				xevent->xkey.state));
+            /*
+             * Set the last event time for use when showing
+             * windows to avoid anti-focus-stealing code.
+             */
+            processing_event = TRUE;
+            last_event_time = xevent->xkey.time;
+            TRACE (g_print ("Current event time %d\n", last_event_time));
 
-		/* 
-		 * Set the last event time for use when showing
-		 * windows to avoid anti-focus-stealing code.
-		 */
-		processing_event = TRUE;
-		last_event_time = xevent->xkey.time;
+            event_mods = xevent->xkey.state & ~(num_lock_mask  |
+                                caps_lock_mask |
+                                scroll_lock_mask);
 
-		event_mods = xevent->xkey.state & ~(num_lock_mask  | 
-						    caps_lock_mask | 
-						    scroll_lock_mask);
+            for (iter = bindings; iter != NULL; iter = iter->next) {
+                Binding *binding = (Binding *) iter->data;
 
-		for (iter = bindings; iter != NULL; iter = iter->next) {
-			Binding *binding = (Binding *) iter->data;
-						       
-			if (binding->keycode == xevent->xkey.keycode &&
-			    binding->modifiers == event_mods) {
+                if (binding->keycode == xevent->xkey.keycode &&
+                    binding->modifiers == event_mods) {
 
-				TRACE (g_print ("Calling handler for '%s'...\n", 
-						binding->keystring));
+                    TRACE (g_print ("Calling handler for '%s'...\n",
+                            binding->keystring));
 
-				(binding->handler) (binding->keystring, 
-						    binding->user_data);
-			}
-		}
+                    (binding->handler) (binding->keystring,
+                                binding->user_data);
+                }
+            }
 
-		processing_event = FALSE;
-		break;
-	case KeyRelease:
-		TRACE (g_print ("Got KeyRelease! \n"));
-		break;
+            processing_event = FALSE;
+            break;
+        case KeyRelease:
+            TRACE (g_print ("Got KeyRelease! \n"));
+            break;
+        default:
+            break;
 	}
 
-	return return_val;
+	return GDK_FILTER_CONTINUE;
 }
 
-static void 
-keymap_changed (GdkKeymap *map)
+static void
+keymap_changed (G_GNUC_UNUSED GdkKeymap *map)
 {
 	GdkKeymap *keymap = gdk_keymap_get_default ();
 	GSList *iter;
@@ -215,7 +229,7 @@ keymap_changed (GdkKeymap *map)
 	}
 }
 
-void 
+void
 tomboy_keybinder_init (void)
 {
 	GdkKeymap *keymap = gdk_keymap_get_default ();
@@ -223,11 +237,11 @@ tomboy_keybinder_init (void)
 
 	lookup_ignorable_modifiers (keymap);
 
-	gdk_window_add_filter (rootwin, 
-			       filter_func, 
+	gdk_window_add_filter (rootwin,
+			       filter_func,
 			       NULL);
 
-	g_signal_connect (keymap, 
+	g_signal_connect (keymap,
 			  "keys_changed",
 			  G_CALLBACK (keymap_changed),
 			  NULL);
@@ -260,7 +274,7 @@ tomboy_keybinder_bind (const char           *keystring,
 }
 
 void
-tomboy_keybinder_unbind (const char           *keystring, 
+tomboy_keybinder_unbind (const char           *keystring,
 			 TomboyBindkeyHandler  handler)
 {
 	GSList *iter;
@@ -269,7 +283,7 @@ tomboy_keybinder_unbind (const char           *keystring,
 		Binding *binding = (Binding *) iter->data;
 
 		if (strcmp (keystring, binding->keystring) != 0 ||
-		    handler != binding->handler) 
+		    handler != binding->handler)
 			continue;
 
 		do_ungrab_key (binding);
@@ -282,7 +296,7 @@ tomboy_keybinder_unbind (const char           *keystring,
 	}
 }
 
-/* 
+/*
  * From eggcellrenderkeys.c.
  */
 gboolean
@@ -314,7 +328,7 @@ tomboy_keybinder_is_modifier (guint keycode)
 guint32
 tomboy_keybinder_get_current_event_time (void)
 {
-	if (processing_event) 
+	if (processing_event)
 		return last_event_time;
 	else
 		return GDK_CURRENT_TIME;
